@@ -29,11 +29,22 @@
 #include <ctype.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#ifdef C_WINDOWS
+#include <sys/utime.h>
+#else
 #include <sys/wait.h>
 #include <utime.h>
+#endif
+#ifdef HAVE_GRP_H
 #include <grp.h>
+#endif
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
 #include <fcntl.h>
+#ifdef	HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <sys/types.h>
 #include <signal.h>
 #include <errno.h>
@@ -44,7 +55,6 @@
 #include "global.h"
 
 #include "shared/options.h"
-#include "shared/memory.h"
 #include "shared/output.h"
 #include "shared/misc.h"
 
@@ -57,6 +67,15 @@
 dev_t procdev;
 #endif
 
+#ifdef C_WINDOWS
+#undef P_tmpdir
+#define P_tmpdir    "C:\\WINDOWS\\TEMP"
+#endif
+
+#ifndef	O_BINARY
+#define	O_BINARY    0
+#endif
+
 static int scandirs(const char *dirname, struct cl_engine *engine, const struct passwd *user, const struct optstruct *opt, const struct cl_limits *limits, int options)
 {
     return treewalk(dirname, engine, user, opt, limits, options, 1);
@@ -132,7 +151,7 @@ int scanmanager(const struct optstruct *opt)
 
 
 /* njh@bandsman.co.uk: BeOS */
-#if !defined(C_CYGWIN) && !defined(C_OS2) && !defined(C_BEOS)
+#if !defined(C_CYGWIN) && !defined(C_OS2) && !defined(C_BEOS) && !defined(C_WINDOWS)
     if(!geteuid()) {
 	if((user = getpwnam(CLAMAVUSER)) == NULL) {
 	    logg("!Can't get information about user "CLAMAVUSER"\n");
@@ -209,7 +228,7 @@ int scanmanager(const struct optstruct *opt)
 	char *cpy, *ptr;
 	ptr = opt_arg(opt, "max-space");
 	if(tolower(ptr[strlen(ptr) - 1]) == 'm') {
-	    cpy = mcalloc(strlen(ptr), sizeof(char));
+	    cpy = calloc(strlen(ptr), 1);
 	    strncpy(cpy, ptr, strlen(ptr) - 1);
 	    limits.maxfilesize = atoi(cpy) * 1024 * 1024;
 	    free(cpy);
@@ -269,6 +288,11 @@ int scanmanager(const struct optstruct *opt)
     else
 	options |= CL_SCAN_OLE2;
 
+    if(opt_check(opt, "no-pdf"))
+	options &= ~CL_SCAN_PDF;
+    else
+	options |= CL_SCAN_PDF;
+
     if(opt_check(opt, "no-html"))
 	options &= ~CL_SCAN_HTML;
     else
@@ -331,7 +355,7 @@ int scanmanager(const struct optstruct *opt)
 			logg("!Can't get absolute pathname of current working directory\n");
 			return 57;
 		    } else {
-			fullpath = mcalloc(512, sizeof(char));
+			fullpath = malloc(512);
 #ifdef NO_SNPRINTF
 			sprintf(fullpath, "%s/%s", cwd, thefilename);
 #else
@@ -383,6 +407,15 @@ int scanmanager(const struct optstruct *opt)
  * -3 -> external signal
  * 0 -> OK
  */
+
+#ifdef C_WINDOWS
+static int clamav_unpack(const char *prog, char **args, const char *tmpdir, const struct passwd *user, const struct optstruct *opt)
+{
+    /* TODO: use spamvp(P_WAIT, prog, args); */
+    cli_errmsg("clamav_unpack is not supported under Windows yet\n");
+    return -1;
+}
+#else
 static int clamav_unpack(const char *prog, char **args, const char *tmpdir, const struct passwd *user, const struct optstruct *opt)
 {
 	pid_t pid;
@@ -400,7 +433,7 @@ static int clamav_unpack(const char *prog, char **args, const char *tmpdir, cons
 	    char *cpy, *ptr;
 	ptr = opt_arg(opt, "max-space");
 	if(tolower(ptr[strlen(ptr) - 1]) == 'm') { /* megabytes */
-	    cpy = mcalloc(strlen(ptr), sizeof(char));
+	    cpy = calloc(strlen(ptr), 1);
 	    strncpy(cpy, ptr, strlen(ptr) - 1);
 	    maxspace = atoi(cpy) * 1024;
 	    free(cpy);
@@ -490,6 +523,7 @@ static int clamav_unpack(const char *prog, char **args, const char *tmpdir, cons
 
     return 0;
 }
+#endif
 
 static void move_infected(const char *filename, const struct optstruct *opt)
 {
@@ -519,8 +553,8 @@ static void move_infected(const char *filename, const struct optstruct *opt)
 
     movefilename_size = sizeof(char) * (strlen(movedir) + strlen(tmp) + sizeof(numext) + 2);
 
-    if(!(movefilename = mmalloc(movefilename_size))) {
-        logg("!mmalloc() failed\n");
+    if(!(movefilename = malloc(movefilename_size))) {
+        logg("!malloc() failed\n");
 	exit(71);
     }
 
@@ -605,7 +639,7 @@ static int checkfile(const char *filename, const struct cl_engine *engine, const
 
     logg("*Scanning %s\n", filename);
 
-    if((fd = open(filename, O_RDONLY)) == -1) {
+    if((fd = open(filename, O_RDONLY|O_BINARY)) == -1) {
 	logg("^Can't open file %s\n", filename);
 	return 54;
     }
@@ -666,7 +700,8 @@ static int scancompressed(const char *filename, struct cl_engine *engine, const
 	exit(63); /* critical */
     }
 
-#ifndef C_OS2
+#if !defined(C_OS2) && !defined(C_WINDOWS)
+    /* FIXME: do the correct native windows way */
     if(user)
 	chown(gendir, user->pw_uid, user->pw_gid);
 #endif
@@ -864,7 +899,7 @@ static int scandenied(const char *filename, struct cl_engine *engine, const stru
 	exit(63); /* critical */
     }
 
-    tmpfile = (char *) mcalloc(strlen(gendir) + strlen(filename) + 10, sizeof(char));
+    tmpfile = (char *) malloc(strlen(gendir) + strlen(filename) + 10);
     pt = strrchr(filename, '/');
     if(!pt)
 	pt = (char *) filename;
@@ -881,7 +916,7 @@ static int scandenied(const char *filename, struct cl_engine *engine, const stru
 
     fixperms(gendir);
 
-#ifndef C_OS2
+#if !defined(C_OS2) && !defined(C_WINDOWS)
     if(user) {
 	chown(gendir, user->pw_uid, user->pw_gid);
 	chown(tmpfile, user->pw_uid, user->pw_gid);
@@ -965,12 +1000,14 @@ int scanfile(const char *filename, struct cl_engine *engine, const struct passwd
 	return 0;
     }
 
+#ifndef C_WINDOWS
     if(geteuid())
 	if(checkaccess(filename, NULL, R_OK) != 1) {
 	    if(!printinfected)
 		logg("%s: Access denied\n", filename);
 	    return 0;
 	}
+#endif
 
     info.files++;
 
