@@ -4,6 +4,7 @@
  *  Proxy support by Nigel Horne <njh@bandsman.co.uk>
  *  Proxy authorization support by Gernot Tenchio <g.tenchio@telco-tech.de>
  *		     (uses fmt_base64() from libowfat (http://www.fefe.de))
+ *  CDIFF code (C) 2006 Sensory Networks, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -50,7 +51,6 @@
 #include <sys/stat.h>
 #include <errno.h>
 
-#include "defaults.h"
 #include "manager.h"
 #include "notify.h"
 #include "dns.h"
@@ -68,6 +68,7 @@
 #include "libclamav/others.h"
 #include "libclamav/str.h"
 #include "libclamav/cvd.h"
+#include "libclamav/lockdb.h"
 
 #ifndef	O_BINARY
 #define	O_BINARY	0
@@ -77,20 +78,9 @@
 #define	closesocket(s)	close(s)
 #endif
 
-static int wwwconnect(const char *server, const char *proxy, int pport, char *ip, const char *localip, int ctimeout, struct mirdat *mdat)
+static int getclientsock(const char *localip)
 {
-	int socketfd = -1, port, i, ret;
-	struct sockaddr_in name;
-	struct hostent *host;
-	char ipaddr[16];
-	unsigned char *ia;
-	const char *hostpt;
-	struct hostent *he = NULL;
-
-    if(ip)
-	strcpy(ip, "???");
-
-    name.sin_family = AF_INET;
+	int socketfd = -1;
 
 #ifdef PF_INET
     socketfd = socket(PF_INET, SOCK_STREAM, 0);
@@ -104,6 +94,8 @@ static int wwwconnect(const char *server, const char *proxy, int pport, char *ip
     }
 
     if(localip) {
+	struct hostent *he;
+
 	if((he = gethostbyname(localip)) == NULL) {
 	    const char *herr;
 	    switch(h_errno) {
@@ -131,6 +123,9 @@ static int wwwconnect(const char *server, const char *proxy, int pport, char *ip
 	    logg("^Using standard local ip address and port for fetching.\n");
 	} else {
 	    struct sockaddr_in client;
+	    unsigned char *ia;
+	    char ipaddr[16];
+
 	    memset ((char *) &client, 0, sizeof(struct sockaddr_in));
 	    client.sin_family = AF_INET;
 	    client.sin_addr = *(struct in_addr *) he->h_addr_list[0];
@@ -144,6 +139,28 @@ static int wwwconnect(const char *server, const char *proxy, int pport, char *ip
 	    }
 	}
     }
+
+    return socketfd;
+}
+
+static int wwwconnect(const char *server, const char *proxy, int pport, char *ip, const char *localip, int ctimeout, struct mirdat *mdat)
+{
+	int socketfd = -1, port, i, ret;
+	struct sockaddr_in name;
+	struct hostent *host;
+	char ipaddr[16];
+	unsigned char *ia;
+	const char *hostpt;
+
+    if(ip)
+	strcpy(ip, "???");
+
+    socketfd = getclientsock(localip);
+    if(socketfd < 0)
+	return -1;
+
+    name.sin_family = AF_INET;
+
     if(proxy) {
 	hostpt = proxy;
 
@@ -170,7 +187,7 @@ static int wwwconnect(const char *server, const char *proxy, int pport, char *ip
     }
 
     if((host = gethostbyname(hostpt)) == NULL) {
-	char *herr;
+	const char *herr;
 	switch(h_errno) {
 	    case HOST_NOT_FOUND:
 		herr = "Host not found";
@@ -225,6 +242,10 @@ static int wwwconnect(const char *server, const char *proxy, int pport, char *ip
 	if(connect(socketfd, (struct sockaddr *) &name, sizeof(struct sockaddr_in)) == -1) {
 #endif
 	    logg("Can't connect to port %d of host %s (IP: %s)\n", port, hostpt, ipaddr);
+	    close(socketfd);
+	    if((socketfd = getclientsock(localip)) == -1)
+		return -1;
+
 	    continue;
 	} else {
 	    mdat->currip = ((struct in_addr *) ia)->s_addr;
@@ -238,10 +259,10 @@ static int wwwconnect(const char *server, const char *proxy, int pport, char *ip
 
 static int Rfc2822DateTime(char *buf, time_t mtime)
 {
-	struct tm *time;
+	struct tm *gmt;
 
-    time = gmtime(&mtime);
-    return strftime(buf, 36, "%a, %d %b %Y %X GMT", time);
+    gmt = gmtime(&mtime);
+    return strftime(buf, 36, "%a, %d %b %Y %X GMT", gmt);
 }
 
 static unsigned int fmt_base64(char *dest, const char *src, unsigned int len)
@@ -312,7 +333,7 @@ static char *proxyauth(const char *user, const char *pass)
     return auth;
 }
 
-static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, char *ip, const char *localip, const char *proxy, int port, const char *user, const char *pass, const char *uas, unsigned int *ims, int ctimeout, int rtimeout, struct mirdat *mdat)
+static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, char *ip, const char *localip, const char *proxy, int port, const char *user, const char *pass, const char *uas, int *ims, int ctimeout, int rtimeout, struct mirdat *mdat)
 {
 	char cmd[512], head[513], buffer[FILEBUFF], ipaddr[16], *ch, *tmp;
 	int bread, cnt, sd;
@@ -627,10 +648,13 @@ static int getfile(const char *srcfile, const char *destfile, const char *hostna
 	    return 57; /* FIXME */
 	}
 
+        if(totalsize > 0) {
+	    totaldownloaded += bread;
+            percentage = (int) (100 * (float) totaldownloaded / totalsize);
+	}
+
         if(!mprintf_quiet) {
             if(totalsize > 0) {
-                totaldownloaded += bread;
-                percentage = (int) (100 * (float) totaldownloaded / totalsize);
                 mprintf("Downloading %s [%3i%%]\r", srcfile, percentage);
             } else {
                 mprintf("Downloading %s [%c]\r", srcfile, rotation[rot]);
@@ -644,7 +668,7 @@ static int getfile(const char *srcfile, const char *destfile, const char *hostna
     close(fd);
 
     if(totalsize > 0)
-        logg("Downloading %s [%i%]\n", srcfile, percentage);
+        logg("Downloading %s [%i%%]\n", srcfile, percentage);
     else
         logg("Downloading %s [*]\n", srcfile);
 
@@ -818,12 +842,12 @@ static struct cl_cvd *currentdb(const char *dbname, unsigned int *inc)
 static int updatedb(const char *dbname, const char *hostname, char *ip, int *signo, const struct cfgstruct *copt, const char *dnsreply, char *localip, int outdated, struct mirdat *mdat)
 {
 	struct cl_cvd *current, *remote;
-	struct cfgstruct *cpt;
-	unsigned int nodb = 0, currver = 0, newver = 0, port = 0, ims = 0, i, j;
-	int ret;
+	const struct cfgstruct *cpt;
+	unsigned int nodb = 0, currver = 0, newver = 0, port = 0, i, j;
+	int ret, ims = -1;
 	char *pt, dbfile[32], dbinc[32], *bacinc = NULL;
 	const char *proxy = NULL, *user = NULL, *pass = NULL, *uas = NULL;
-	unsigned int flevel = cl_retflevel(), inc;
+	unsigned int flevel = cl_retflevel(), inc, maxattempts;
 	struct stat sb;
 	int ctimeout, rtimeout;
 
@@ -991,8 +1015,9 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
 	    }
 	}
 
+	maxattempts = cfgopt(copt, "MaxAttempts")->numarg;
 	for(i = currver + 1; i <= newver; i++) {
-	    for(j = 0; j < cfgopt(copt, "MaxAttempts")->numarg; j++) {
+	    for(j = 0; j < maxattempts; j++) {
 		ret = getpatch(dbname, i, hostname, ip, localip, proxy, port, user, pass, uas, ctimeout, rtimeout, mdat);
 		if(ret == 52 || ret == 58) {
 		    memset(ip, 0, 16);
@@ -1055,14 +1080,14 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
     return 0;
 }
 
-int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, const char *hostname)
+int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, const char *hostname, const char *dbdir)
 {
 	time_t currtime;
 	int ret, updated = 0, outdated = 0, signo = 0;
-	unsigned int ttl;
+	unsigned int ttl, try = 0;
 	char ipaddr[16], *dnsreply = NULL, *pt, *localip = NULL, *newver = NULL;
 	const char *arg = NULL;
-	struct cfgstruct *cpt;
+	const struct cfgstruct *cpt;
 	struct mirdat mdat;
 #ifdef HAVE_RESOLV_H
 	const char *dnsdbinfo;
@@ -1147,6 +1172,19 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 	localip = cpt->strarg;
     }
 
+    while(cli_writelockdb(dbdir, 0) == CL_ELOCKDB) {
+	logg("*Waiting to lock database directory: %s\n", dbdir);
+	sleep(5);
+	if(++try > 12) {
+	    logg("!Can't lock database directory: %s\n", dbdir);
+	    if(dnsreply)
+		free(dnsreply);
+	    if(newver)
+		free(newver);
+	    return 61; 
+	}
+    }
+
     if(cfgopt(copt, "HTTPProxyServer")->enabled)
 	mirman_read("mirrors.dat", &mdat, 0);
     else
@@ -1162,6 +1200,7 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 	    free(newver);
 
 	mirman_write("mirrors.dat", &mdat);
+	cli_unlockdb(dbdir);
 	return ret;
 
     } else if(ret == 0)
@@ -1176,6 +1215,7 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 	    free(newver);
 
 	mirman_write("mirrors.dat", &mdat);
+	cli_unlockdb(dbdir);
 	return ret;
 
     } else if(ret == 0)
@@ -1185,6 +1225,7 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 	free(dnsreply);
 
     mirman_write("mirrors.dat", &mdat);
+    cli_unlockdb(dbdir);
 
     if(updated) {
 	if(cfgopt(copt, "HTTPProxyServer")->enabled) {
@@ -1235,7 +1276,7 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 		    free(cmd);
 		    if(newver)
 			free(newver);
-		    return 0;
+		    return 75;
 		}
 
 		*pt = 0; pt += 2;
