@@ -13,17 +13,18 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ *  MA 02110-1301, USA.
  */
 
 #ifndef __FSG_H
 #define __FSG_H
 
 #include "cltypes.h"
-#include "rebuildpe.h"
+#include "execs.h"
 
 int unfsg_200(char *, char *, int, int, uint32_t, uint32_t, uint32_t, int);
-int unfsg_133(char *, char *, int , int, struct SECTION *, int, uint32_t, uint32_t, int);
+int unfsg_133(char *, char *, int , int, struct cli_exe_section *, int, uint32_t, uint32_t, int);
 
 #endif
 
