@@ -1,7 +1,7 @@
 /*
  *  HTML Entity & Encoding normalization.
  *
- *  Copyright (C) 2006 Trk Edvin <edwin@clamav.net>
+ *  Copyright (C) 2006-2007 Trk Edvin <edwin@clamav.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include "others.h"
 #include "hashtab.h"
 
+
 static const size_t prime_list[] =
 {
      53ul,         97ul,         193ul,       389ul,       769ul,
@@ -80,13 +81,13 @@ static inline void PROFILE_FIND_ELEMENT(struct hashtable *s)
 static inline void PROFILE_FIND_NOTFOUND(struct hashtable *s, size_t tries)
 {
 	s->PROFILE_STRUCT.not_found++;
-	s->PROFILE_STRUCT.not_found_tries += tries+1;
+	s->PROFILE_STRUCT.not_found_tries += tries;
 }
 
 static inline void PROFILE_FIND_FOUND(struct hashtable *s, size_t tries)
 {
 	s->PROFILE_STRUCT.found++;
-	s->PROFILE_STRUCT.found_tries += tries+1;
+	s->PROFILE_STRUCT.found_tries += tries;
 }
 
 static inline void PROFILE_HASH_EXHAUSTED(struct hashtable *s)
@@ -102,7 +103,7 @@ static inline void PROFILE_GROW_START(struct hashtable *s)
 static inline void PROFILE_GROW_FOUND(struct hashtable *s, size_t tries)
 {
 	s->PROFILE_STRUCT.grow_found++;
-	s->PROFILE_STRUCT.grow_found_tries += tries+1;
+	s->PROFILE_STRUCT.grow_found_tries += tries;
 }
 
 static inline void PROFILE_GROW_DONE(struct hashtable *s)
@@ -112,19 +113,19 @@ static inline void PROFILE_GROW_DONE(struct hashtable *s)
 static inline void PROFILE_DELETED_REUSE(struct hashtable *s, size_t tries)
 {
 	s->PROFILE_STRUCT.deleted_reuse++;
-	s->PROFILE_STRUCT.deleted_tries += tries+1;
+	s->PROFILE_STRUCT.deleted_tries += tries;
 }
 
 static inline void PROFILE_INSERT(struct hashtable *s, size_t tries)
 {
 	s->PROFILE_STRUCT.inserts++;
-	s->PROFILE_STRUCT.insert_tries += tries+1;
+	s->PROFILE_STRUCT.insert_tries += tries;
 }
 
 static inline void PROFILE_DATA_UPDATE(struct hashtable *s, size_t tries)
 {
 	s->PROFILE_STRUCT.update++;
-	s->PROFILE_STRUCT.update_tries += tries+1;
+	s->PROFILE_STRUCT.update_tries += tries;
 }
 
 static inline void PROFILE_HASH_DELETE(struct hashtable *s)
@@ -140,7 +141,7 @@ static inline void PROFILE_HASH_CLEAR(struct hashtable *s)
 static inline void PROFILE_REPORT(const struct hashtable *s)
 {
 	size_t lookups, queries, insert_tries, inserts;
-	cli_dbgmsg("--------Hashtable usage report for %x--------------\n",s);
+	cli_dbgmsg("--------Hashtable usage report for %p--------------\n",(const void*)s);
 	cli_dbgmsg("hash function calculations:%ld\n",s->PROFILE_STRUCT.calc_hash);
 	cli_dbgmsg("successfull finds/total searches: %ld/%ld; lookups: %ld\n", s->PROFILE_STRUCT.found, s->PROFILE_STRUCT.find_req, s->PROFILE_STRUCT.found_tries);
 	cli_dbgmsg("unsuccessfull finds/total searches: %ld/%ld; lookups: %ld\n", s->PROFILE_STRUCT.not_found, s->PROFILE_STRUCT.find_req , s->PROFILE_STRUCT.not_found_tries);
@@ -195,7 +196,7 @@ int hashtab_init(struct hashtable *s,size_t capacity)
 	return 0;
 }
 
-static inline size_t hash(const unsigned char* k,const size_t len,const size_t SIZE)
+static size_t hash(const unsigned char* k,const size_t len,const size_t SIZE)
 {
 	size_t Hash = 0;	
 	size_t i;
@@ -208,7 +209,7 @@ static inline size_t hash(const unsigned char* k,const size_t len,const size_t S
 struct element* hashtab_find(const struct hashtable *s,const unsigned char* key,const size_t len)
 {
 	struct element* element;
-	size_t tries = 0; 
+	size_t tries = 1; 
 	size_t idx;
 
 	if(!s)
@@ -248,10 +249,10 @@ static int hashtab_grow(struct hashtable *s)
 	for(i=0; i < s->capacity;i++) {
 		if(s->htable[i].key && s->htable[i].key != DELETED_KEY) {
 			struct element* element;
-			size_t tries = 0;
+			size_t tries = 1;				
 
 			PROFILE_CALC_HASH(s);
-			idx = hash(s->htable[i].key, strlen((const char*)s->htable[i].key), s->capacity);
+			idx = hash(s->htable[i].key, strlen((const char*)s->htable[i].key), new_capacity);
 			element = &htable[idx];
 
 			while(element->key && tries <= new_capacity) {
@@ -275,7 +276,7 @@ static int hashtab_grow(struct hashtable *s)
 	s->used = used;
 	s->capacity = new_capacity;
 	s->maxfill = new_capacity*8/10;
-	cli_dbgmsg("Table %x size after grow:%ld\n",s,s->capacity);
+	cli_dbgmsg("Table %p size after grow:%ld\n",(void*)s,s->capacity);
 	PROFILE_GROW_DONE(s);
 	return CL_SUCCESS;
 }
@@ -285,7 +286,7 @@ int hashtab_insert(struct hashtable *s,const unsigned char* key,const size_t len
 {
 	struct element* element;
 	struct element* deleted_element = NULL;
-	size_t tries = 0; 
+	size_t tries = 1; 
 	size_t idx;
 	if(!s)
 		return CL_ENULLARG; 
@@ -309,12 +310,12 @@ int hashtab_insert(struct hashtable *s,const unsigned char* key,const size_t len
 				thekey = cli_malloc(len+1);
 				if(!thekey)
 					return CL_EMEM;
-				strncpy((char*)thekey,(const char*)key,len);
+				strncpy((char*)thekey,(const char*)key,len+1);
 				element->key = thekey;
 				element->data = data;
 				s->used++;		
 				if(s->used > s->maxfill) {
-					cli_dbgmsg("hashtab.c:Growing hashtable %x, because it has exceeded maxfill, old size:%ld\n",s->capacity);
+					cli_dbgmsg("hashtab.c:Growing hashtable %p, because it has exceeded maxfill, old size:%ld\n",(void*)s,s->capacity);
 					hashtab_grow(s);
 				}
 				return 0;
@@ -334,7 +335,7 @@ int hashtab_insert(struct hashtable *s,const unsigned char* key,const size_t len
 		} while (tries <= s->capacity);
 		/* no free place found*/
 		PROFILE_HASH_EXHAUSTED(s);
-		cli_dbgmsg("hashtab.c: Growing hashtable %x, because its full, old size:%ld.\n",s,s->capacity);
+		cli_dbgmsg("hashtab.c: Growing hashtable %p, because its full, old size:%ld.\n",(void*)s,s->capacity);
 	} while( hashtab_grow(s) >= 0 );
 	cli_warnmsg("hashtab.c: Unable to grow hashtable\n");
 	return CL_EMEM;
@@ -400,8 +401,6 @@ int hashtab_generate_c(const struct hashtable *s,const char* name)
 	return 0;
 }
 
-
-
 int hashtab_load(FILE* in, struct hashtable *s)
 {
 	char line[1024];
