@@ -49,9 +49,6 @@
 #include "others.h"
 #include "ole2_extract.h"
 
-#include "mbox.h"
-#include "blob.h" /* sanitiseName() */
-
 #define ole2_endian_convert_16(v) le16_to_host(v)
 #define ole2_endian_convert_32(v) le32_to_host(v)
 
@@ -63,6 +60,10 @@
 #pragma pack(1)
 #endif
 
+#ifdef HAVE_PRAGMA_PACK_HPPA
+#pragma pack 1
+#endif
+
 #ifndef	O_BINARY
 #define	O_BINARY	0
 #endif
@@ -126,6 +127,10 @@ typedef struct property_tag
 #pragma pack()
 #endif
 
+#ifdef HAVE_PRAGMA_PACK_HPPA
+#pragma pack
+#endif
+
 static unsigned char magic_id[] = { 0xd0, 0xcf, 0x11, 0xe0, 0xa1, 0xb1, 0x1a, 0xe1};
 
 static char *get_property_name(char *name, int size)
@@ -603,7 +608,7 @@ static int handler_writefile(int fd, ole2_header_t *hdr, property_t *prop, const
 		snprintf(name, 11, "%.10ld", i + (long int) prop);
 	} else {
 		/* Sanitize the file name */
-		sanitiseName(name);
+		cli_sanitise_filename(name);
 	}
 
 	newname = (char *) cli_malloc(strlen(name) + strlen(dir) + 2);
@@ -673,7 +678,7 @@ static int handler_writefile(int fd, ole2_header_t *hdr, property_t *prop, const
 	return TRUE;
 }
 
-#if !defined(HAVE_ATTRIB_PACKED) && !defined(HAVE_PRAGMA_PACK)
+#if !defined(HAVE_ATTRIB_PACKED) && !defined(HAVE_PRAGMA_PACK) && !defined(HAVE_PRAGMA_PACK_HPPA)
 static int ole2_read_header(int fd, ole2_header_t *hdr)
 {
 	int i;
@@ -767,7 +772,7 @@ int cli_ole2_extract(int fd, const char *dirname, const struct cl_limits *limits
 #endif
 
 	if (hdr.m_area == NULL) {
-#if defined(HAVE_ATTRIB_PACKED) || defined(HAVE_PRAGMA_PACK)
+#if defined(HAVE_ATTRIB_PACKED) || defined(HAVE_PRAGMA_PACK) || defined(HAVE_PRAGMA_PACK_HPPA)
 		if (cli_readn(fd, &hdr, hdr_size) != hdr_size) {
 			return 0;
 		}
