@@ -71,10 +71,9 @@ static	const	char	*cli_pmemstr(const char *haystack, size_t hs, const char *need
 int
 cli_pdf(const char *dir, int desc, const cli_ctx *ctx)
 {
-	struct stat statb;
 	off_t size;	/* total number of bytes in the file */
 	long bytesleft, trailerlength;
-	char *buf;	/* start of memory mapped area */
+	char *buf, *alloced;	/* start of memory mapped area */
 	const char *p, *q, *trailerstart;
 	const char *xrefstart;	/* cross reference table */
 	/*size_t xreflength;*/
@@ -82,6 +81,7 @@ cli_pdf(const char *dir, int desc, const cli_ctx *ctx)
 	struct table *md5table;
 	int printed_predictor_message;
 	int printed_embedded_font_message;
+	struct stat statb;
 
 	cli_dbgmsg("in cli_pdf(%s)\n", dir);
 
@@ -100,20 +100,37 @@ cli_pdf(const char *dir, int desc, const cli_ctx *ctx)
 	if(buf == MAP_FAILED)
 		return CL_EMEM;
 
+	alloced = cli_malloc(size);
+	if(alloced) {
+		/*
+		 * FIXME: now I have this, there's no need for the lack of
+		 *	support on systems without mmap, e.g. cygwin
+		 */
+		memcpy(alloced, buf, size);
+		munmap(buf, size);
+		p = alloced;
+	}
+
 	cli_dbgmsg("cli_pdf: scanning %lu bytes\n", (unsigned long)size);
 
 	/* Lines are terminated by \r, \n or both */
 
 	/* File Header */
 	if(memcmp(p, "%PDF-1.", 7) != 0) {
-		munmap(buf, size);
+		if(alloced)
+			free(alloced);
+		else
+			munmap(buf, size);
 		return CL_EFORMAT;
 	}
 
 #if	0
 	q = pdf_nextlinestart(&p[6], size - 6);
 	if(q == NULL) {
-		munmap(buf, size);
+		if(alloced)
+			free(alloced);
+		else
+			munmap(buf, size);
 		return CL_EFORMAT;
 	}
 	bytesleft = size - (long)(q - p);
@@ -128,8 +145,11 @@ cli_pdf(const char *dir, int desc, const cli_ctx *ctx)
 		if(memcmp(q, "%%EOF", 5) == 0)
 			break;
 
-	if(q == p) {
-		munmap(buf, size);
+	if(q <= p) {
+		if(alloced)
+			free(alloced);
+		else
+			munmap(buf, size);
 		return CL_EFORMAT;
 	}
 
@@ -145,7 +165,10 @@ cli_pdf(const char *dir, int desc, const cli_ctx *ctx)
 		/*
 		 * This tends to mean that the file is, in effect, read-only
 		 */
-		munmap(buf, size);
+		if(alloced)
+			free(alloced);
+		else
+			munmap(buf, size);
 		cli_warnmsg("Encrypted PDF files not yet supported\n");
 		return CL_EFORMAT;
 	}
@@ -168,7 +191,10 @@ cli_pdf(const char *dir, int desc, const cli_ctx *ctx)
 				break;
 
 	if(xrefstart == p) {
-		munmap(buf, size);
+		if(alloced)
+			free(alloced);
+		else
+			munmap(buf, size);
 		return CL_EFORMAT;
 	}
 
@@ -185,9 +211,8 @@ cli_pdf(const char *dir, int desc, const cli_ctx *ctx)
 	 * The body section consists of a sequence of indirect objects
 	 */
 	while((p < xrefstart) &&
-	      ((q = pdf_nextobject(p, bytesleft)) != NULL) &&
-	      (rc == CL_CLEAN)) {
-		int is_ascii85decode, is_flatedecode, fout, len;
+	      ((q = pdf_nextobject(p, bytesleft)) != NULL)) {
+		int is_ascii85decode, is_flatedecode, fout, len, has_cr;
 		/*int object_number, generation_number;*/
 		const char *objstart, *objend, *streamstart, *streamend;
 		char *md5digest;
@@ -260,6 +285,36 @@ cli_pdf(const char *dir, int desc, const cli_ctx *ctx)
 					length = atoi(q);
 					while(isdigit(*q))
 						q++;
+					/*
+					 * Note: incremental updates are not
+					 *	supported
+					 */
+					if((bytesleft > 11) && strncmp(q, " 0 R", 4) == 0) {
+						const char *r;
+						char b[13];
+
+						q += 4;
+						cli_dbgmsg("Length is in indirect obj %d\n",
+							length);
+						snprintf(b, sizeof(b),
+							"%d 0 obj", length);
+						length = strlen(b);
+						r = cli_pmemstr(alloced ? alloced : buf,
+							size, b, length);
+						if(r) {
+							r += length - 1;
+							r = pdf_nextobject(r, bytesleft - (r - q));
+							if(r) {
+								length = atoi(r);
+								while(isdigit(*r))
+									r++;
+								cli_dbgmsg("length in '%s' %d\n",
+									b, length);
+							}
+						} else
+							cli_warnmsg("Couldn't find '%s'\n",
+								b);
+					}
 					q--;
 				} else if(strncmp(q, "Length2 ", 8) == 0)
 					is_embedded_font = 1;
@@ -322,7 +377,9 @@ cli_pdf(const char *dir, int desc, const cli_ctx *ctx)
 				cli_dbgmsg("No endstream\n");
 				break;
 			}
-		}
+			has_cr = 1;
+		} else
+			has_cr = 0;
 		snprintf(fullname, sizeof(fullname), "%s/pdfXXXXXX", dir);
 #if	defined(C_LINUX) || defined(C_BSD) || defined(HAVE_MKSTEMP) || defined(C_SOLARIS) || defined(C_CYGWIN)
 		fout = mkstemp(fullname);
@@ -355,14 +412,17 @@ cli_pdf(const char *dir, int desc, const cli_ctx *ctx)
 		 * Calculate the length ourself, the Length parameter is often
 		 * wrong
 		 */
-		while(strchr("\r\n", *--streamend))
-			;
+		if(*--streamend != '\n')
+			streamend++;
+		else if(has_cr && (*--streamend != '\r'))
+			streamend++;
 
 		if(streamend <= streamstart) {
+			close(fout);
 			cli_dbgmsg("Empty stream\n");
 			continue;
 		}
-		calculated_streamlen = (int)(streamend - streamstart) + 1;
+		calculated_streamlen = (int)(streamend - streamstart);
 		real_streamlen = length;
 
 		if(calculated_streamlen != real_streamlen)
@@ -439,7 +499,10 @@ cli_pdf(const char *dir, int desc, const cli_ctx *ctx)
 		cli_dbgmsg("cli_pdf: extracted to %s\n", fullname);
 	}
 
-	munmap(buf, size);
+	if(alloced)
+		free(alloced);
+	else
+		munmap(buf, size);
 
 	tableDestroy(md5table);
 
@@ -456,10 +519,22 @@ try_flatedecode(unsigned char *buf, off_t real_len, off_t calculated_len, int fo
 	if(ret == Z_OK)
 		return Z_OK;
 
-	if(real_len == calculated_len)
+	if(real_len == calculated_len) {
+		/*
+		 * Nothing more we can do to inflate
+		 */
+		cli_warnmsg("Bad compression in flate stream\n");
 		return ret;
+	}
 
-	return flatedecode(buf, calculated_len, fout, ctx);
+	ret = flatedecode(buf, calculated_len, fout, ctx);
+	if(ret == Z_OK)
+		return Z_OK;
+
+	/* i.e. the PDF file is broken :-( */
+	cli_warnmsg("cli_pdf: Bad compressed block length in flate stream\n");
+
+	return ret;
 }
 
 static int
@@ -495,6 +570,8 @@ flatedecode(unsigned char *buf, off_t len, int fout, const cli_ctx *ctx)
 		if(tmpfp) {
 			fwrite(buf, sizeof(char), len, tmpfp);
 			fclose(tmpfp);
+			cli_dbgmsg("cli_pdf: flatedecode: debugging file is %s\n",
+				tmpfilename);
 		} else
 			cli_errmsg("cli_pdf: can't fdopen debugging file\n");
 	}
@@ -515,7 +592,7 @@ flatedecode(unsigned char *buf, off_t len, int fout, const cli_ctx *ctx)
 
 	nbytes = 0;
 
-	for(;;) {
+	while(stream.avail_in) {
 		zstat = inflate(&stream, Z_NO_FLUSH);	/* zlib */
 		switch(zstat) {
 			case Z_OK:
