@@ -13,7 +13,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ *  MA 02110-1301, USA.
  */
 
 #ifndef __STR_H
@@ -26,7 +27,8 @@ short int *cli_hex2si(const char *hex);
 char *cli_hex2str(const char *hex);
 int cli_hex2num(const char *hex);
 char *cli_str2hex(const char *string, unsigned int len);
+char *cli_utf16toascii(const char *str, unsigned int length);
 char *cli_strtokbuf(const char *input, int fieldno, const char *delim, char *output);
 const char *cli_memstr(const char *haystack, int hs, const char *needle, int ns);
-
+char *cli_strrcpy(char *dest, const char *source);
 #endif
