@@ -13,7 +13,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ *  MA 02110-1301, USA.
  */
 
 #include <stdio.h>
@@ -24,24 +25,24 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <dirent.h>
-#include <clamav.h>
 #include <ctype.h>
 
-#include "../libclamav/vba_extract.h"
-#include "../libclamav/others.h"
-#include "../libclamav/cltypes.h"
-#include "../libclamav/ole2_extract.h"
+#include "libclamav/clamav.h"
+#include "libclamav/vba_extract.h"
+#include "libclamav/others.h"
+#include "libclamav/cltypes.h"
+#include "libclamav/ole2_extract.h"
 
 typedef struct mac_token_tag
 {
     unsigned char token;
-    char *str;
+    const unsigned char *str;
 } mac_token_t;
 
 typedef struct mac_token2_tag
 {
     uint16_t token;
-    char *str;
+    const unsigned char *str;
 
 } mac_token2_t;
 
@@ -76,7 +77,7 @@ static char *get_unicode_name (char *name, int size)
     return newname;
 }
 
-void output_token (unsigned char token)
+static void output_token (unsigned char token)
 {
     int i;
     mac_token_t mac_token[] = {
@@ -172,7 +173,7 @@ void output_token (unsigned char token)
     return;
 }
 
-void output_token67 (uint16_t token)
+static void output_token67 (uint16_t token)
 {
     int i;
     mac_token2_t mac_token[] = {
@@ -522,7 +523,7 @@ void output_token67 (uint16_t token)
     return;
 }
 
-void output_token73 (uint16_t token)
+static void output_token73 (uint16_t token)
 {
     int i;
     mac_token2_t mac_token[] = {
@@ -773,7 +774,7 @@ void output_token73 (uint16_t token)
     return;
 }
 
-void print_hex_buff (unsigned char *start, unsigned char *end, int hex_output)
+static void print_hex_buff (unsigned char *start, unsigned char *end, int hex_output)
 {
     if (!hex_output) {
 	return;
@@ -786,7 +787,7 @@ void print_hex_buff (unsigned char *start, unsigned char *end, int hex_output)
     printf ("]\n");
 }
 
-void wm_decode_macro (unsigned char *buff, uint32_t len, int hex_output)
+static void wm_decode_macro (unsigned char *buff, uint32_t len, int hex_output)
 {
     int i, j;
     uint8_t s_length;
@@ -800,7 +801,7 @@ void wm_decode_macro (unsigned char *buff, uint32_t len, int hex_output)
 	case 0x65:
 	    s_length = (uint8_t) buff[i + 1];
 	    tmp_buff = (unsigned char *) malloc (s_length + 1);
-	    strncpy ((char *) tmp_buff, (char *) (buff + i + 2), s_length);
+	    strncpy (tmp_buff, buff + i + 2, s_length);
 	    tmp_buff[s_length] = '\0';
 	    print_hex_buff (line_start, buff + i + 2 + s_length, hex_output);
 	    printf ("\n%s", tmp_buff);
@@ -811,7 +812,7 @@ void wm_decode_macro (unsigned char *buff, uint32_t len, int hex_output)
 	case 0x69:
 	    s_length = (uint8_t) buff[i + 1];
 	    tmp_buff = (unsigned char *) malloc (s_length + 1);
-	    strncpy ((char *) tmp_buff, (char *) (buff + i + 2), s_length);
+	    strncpy (tmp_buff, buff + i + 2, s_length);
 	    tmp_buff[s_length] = '\0';
 	    printf (" %s", tmp_buff);
 	    free (tmp_buff);
@@ -820,7 +821,7 @@ void wm_decode_macro (unsigned char *buff, uint32_t len, int hex_output)
 	case 0x6a:
 	    s_length = (uint8_t) buff[i + 1];
 	    tmp_buff = (unsigned char *) malloc (s_length + 1);
-	    strncpy ((char *) tmp_buff, (char *) (buff + i + 2), s_length);
+	    strncpy (tmp_buff, buff + i + 2, s_length);
 	    tmp_buff[s_length] = '\0';
 	    printf (" \"%s\"", tmp_buff);
 	    free (tmp_buff);
@@ -829,7 +830,7 @@ void wm_decode_macro (unsigned char *buff, uint32_t len, int hex_output)
 	case 0x6b:
 	    s_length = (uint8_t) buff[i + 1];
 	    tmp_buff = (unsigned char *) malloc (s_length + 1);
-	    strncpy ((char *) tmp_buff, (char *) (buff + i + 2), s_length);
+	    strncpy (tmp_buff, buff + i + 2, s_length);
 	    tmp_buff[s_length] = '\0';
 	    printf (" '%s", tmp_buff);
 	    free (tmp_buff);
@@ -838,7 +839,7 @@ void wm_decode_macro (unsigned char *buff, uint32_t len, int hex_output)
 	case 0x6d:
 	    s_length = (uint8_t) buff[i + 1];
 	    tmp_buff = (unsigned char *) malloc (s_length + 1);
-	    strncpy ((char *) tmp_buff, (char *) (buff + i + 2), s_length);
+	    strncpy (tmp_buff, buff + i + 2, s_length);
 	    tmp_buff[s_length] = '\0';
 	    printf (" %s", tmp_buff);
 	    free (tmp_buff);
@@ -847,7 +848,7 @@ void wm_decode_macro (unsigned char *buff, uint32_t len, int hex_output)
 	case 0x70:
 	    s_length = (uint8_t) buff[i + 1];
 	    tmp_buff = (unsigned char *) malloc (s_length + 1);
-	    strncpy ((char *) tmp_buff, (char *) (buff + i + 2), s_length);
+	    strncpy (tmp_buff, buff + i + 2, s_length);
 	    tmp_buff[s_length] = '\0';
 	    printf ("REM%s", tmp_buff);
 	    free (tmp_buff);
@@ -856,7 +857,7 @@ void wm_decode_macro (unsigned char *buff, uint32_t len, int hex_output)
 	case 0x76:
 	    s_length = (uint8_t) buff[i + 1];
 	    tmp_buff = (unsigned char *) malloc (s_length + 1);
-	    strncpy ((char *) tmp_buff, (char *) (buff + i + 2), s_length);
+	    strncpy (tmp_buff, buff + i + 2, s_length);
 	    tmp_buff[s_length] = '\0';
 	    printf (" .%s", tmp_buff);
 	    free (tmp_buff);
@@ -865,7 +866,7 @@ void wm_decode_macro (unsigned char *buff, uint32_t len, int hex_output)
 	case 0x77:
 	    s_length = (uint8_t) buff[i + 1];
 	    tmp_buff = (unsigned char *) malloc (s_length + 1);
-	    strncpy ((char *) tmp_buff, (char *) (buff + i + 2), s_length);
+	    strncpy (tmp_buff, buff + i + 2, s_length);
 	    tmp_buff[s_length] = '\0';
 	    printf ("%s", tmp_buff);
 	    free (tmp_buff);
@@ -875,7 +876,7 @@ void wm_decode_macro (unsigned char *buff, uint32_t len, int hex_output)
 	    w_length = (uint16_t) (buff[i + 2] << 8) + buff[i + 1];
 	    tmp_buff = (unsigned char *) malloc ((w_length * 2) + 1);
 	    memcpy (tmp_buff, buff + i + 3, w_length * 2);
-	    tmp_name = (unsigned char *) get_unicode_name ((char *) tmp_buff, w_length * 2);
+	    tmp_name = get_unicode_name (tmp_buff, w_length * 2);
 	    free (tmp_buff);
 	    printf ("\"%s\"", tmp_name);
 	    free (tmp_name);
@@ -886,7 +887,7 @@ void wm_decode_macro (unsigned char *buff, uint32_t len, int hex_output)
 	    s_length = (uint8_t) buff[i + 1];
 	    tmp_buff = (unsigned char *) malloc ((s_length * 2) + 1);
 	    memcpy (tmp_buff, buff + i + 2, s_length * 2);
-	    tmp_name = (unsigned char *) get_unicode_name ((char *) tmp_buff, s_length * 2);
+	    tmp_name = get_unicode_name (tmp_buff, s_length * 2);
 	    free (tmp_buff);
 	    printf ("'%s", tmp_name);
 	    free (tmp_name);
@@ -1059,7 +1060,7 @@ int sigtool_vba_scandir (const char *dirname, int hex_output)
 	    if (!data) {
 		cli_dbgmsg ("WARNING: VBA project '%s' decompressed to NULL\n", vba_project->name[i]);
 	    } else {
-		data = (unsigned char *) realloc (data, data_len + 1);
+		data = (char *) realloc (data, data_len + 1);
 		data[data_len] = '\0';
 		printf ("%s", data);
 		free (data);
